const path = require('path');
const mustache = require('mustache');
const fs = require('fs');
const parseXmlStringSync = require('xml2js-parser').parseStringSync;
const globby = require('globby');
const _ = require('underscore');
const svgToMiniDataURI = require('mini-svg-data-uri');
const imagemin = require('imagemin');
const imageminSvgo = require('imagemin-svgo');

module.exports = class InlineSvg {

	/**
	 * Kick things off
	 *
	 * @param {String} _svgFilePath The path to the folder containing the SVGs
	 * @param {Object} _options Configuration object
	 * @returns {Promise}
	 */
	constructor(_svgFilePath, _options) {
		let defaultOptions = {
			template: __dirname + '/template.mustache',
			context: {},
			interceptor: null
		};
		this._files = {};
		this._svgs = [];
		this._options = _.extend(defaultOptions, _options);
		return this._process(_svgFilePath);
	}

	/**
	 * Process all SVG files
	 *
	 * @param {String} _svgFilePath The path to the folder containing the SVGs
	 */
	async _process(_svgFilePath) {
		return new Promise(async (_resolve, _reject) => {
			try {
				let svgs = await this._readFiles(_svgFilePath);
				let templateContent = await imagemin.buffer(this._getTemplateContent(), {
					plugins: [
						imageminSvgo()
					]
				});
				svgs.forEach(svg => this._processSvg(svg, _svgFilePath));
				let template = mustache.render(
					templateContent.toString(),
					_.extend(
						{},
						this._options.context,
						{
							svgs: this._svgs
						}
					)
				);
				_resolve(template);
			} catch (e) {
				console.log(e);
			}
		});
	}

	/**
	 * Processes a single SVG icon
	 *
	 * _data is expected to carry two properties:
	 *        data.content: A string containing the SVG source
	 *        data.fileName: A string containing the file name
	 *
	 * @param {Object} _data Object containing info about the SVG
	 * @param {String} _svgFilePath Original File Path from the fetched SVG
	 */
	_processSvg(_data, _svgFilePath) {
		const xmlString = parseXmlStringSync(_data.content.toString(), {
			attrNameProcessors: [name => name.toLowerCase()]
		});
		const svgDimensions = this._getSvgDimensions(xmlString);
		const _svgFilePathAdjusted = _svgFilePath.replace('**', '');
		const svgData = {
			name: _data.fileName.replace(_svgFilePathAdjusted, '').replace(/\//g, '-').replace('.svg', '').replace('fontawesome-', ''),
			// add fill="white" to allow later color changes based on this value
			inline: svgToMiniDataURI(_data.content.indexOf('fill=') === -1 ? _data.content.replace('<svg', '<svg fill="white"') : _data.content),
			width: parseInt(svgDimensions.width) + 'px',
			height: parseInt(svgDimensions.height) + 'px',
			dimensions: svgDimensions
		};

		// store this svg data
		this._svgs.push(
			_.isFunction(this._options.interceptor)
				? _.extend({}, svgData, this._options.interceptor(svgData) || svgData)
				: svgData
		);
	}

	/**
	 * Extracts the dimensions from the SVG source
	 *
	 * @param {String} xmlString The SVG source
	 * @returns {Object}
	 */
	_getSvgDimensions(xmlString) {
		const hasWidthHeightAttr = xmlString.svg.$['width'] && xmlString.svg.$['width'];
		let width;
		let height;
		if (hasWidthHeightAttr) {
			width = parseInt(xmlString.svg.$['width']);
			height = parseInt(xmlString.svg.$['height']);
		} else {
			width = parseInt(xmlString.svg.$['viewbox'].toString().replace(/^\d+\s\d+\s(\d+\.?[\d])\s(\d+\.?[\d])/, "$1"));
			height = parseInt(xmlString.svg.$['viewbox'].toString().replace(/^\d+\s\d+\s(\d+\.?[\d])\s(\d+\.?[\d])/, "$2"));
		}
		return {
			width,
			height
		};
	}

	/**
	 * Reads all the SVGs that are located in _path
	 *
	 * @param {String} _path The location to read from
	 * @returns {Promise}
	 */
	async _readFiles(_path) {
		return new Promise(async (resolve, reject) => {
			let files = await globby(path.join(_path, '*.svg'));
			let svgs = [];
			await files.forEach(async _file => {
				let content = fs.readFileSync(_file, 'utf-8');
				svgs.push({
					fileName: _file,
					content: content
				});
			});
			resolve(svgs);
		})
	}

	/**
	 * Returns the mustache template as a string
	 *
	 * @returns {String}
	 */
	_getTemplateContent() {
		return fs.readFileSync(this._options.template);
	}
};
